;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "centered-window" "20250921.2101"
  "Center the text when there's only one window."
  '((emacs "24.4"))
  :url "https://github.com/nullvec/centered-window-mode"
  :commit "701f56cd1d2b68352d29914f05ca1b0037bb2595"
  :revdesc "701f56cd1d2b"
  :keywords '("faces" "windows")
  :authors '(("A. Hdez" . "trefoil_chilled_7k@icloud.com"))
  :maintainers '(("A. Hdez" . "trefoil_chilled_7k@icloud.com")))
